/**
   Example of 3 periodic tasks scheduled by fixed priority, using
   clock_nanosleep()
 */

#include <stdio.h>
#include <pthread.h>
#include <unistd.h>
#include <time.h>
#include <stdlib.h>
#include "time_utils.h"

struct periodic_data_struct {
  int index;
  long period_us;
  int wcet_sim;
};


#define NTHREADS 3

void *thread_code(void *arg) {
  struct periodic_data_struct *ps = (struct periodic_data_struct *) arg;
  int j;
  int a = 13, b = 17; 
  struct timespec next;
  struct timespec now;

  clock_gettime(CLOCK_REALTIME, &next);
  while (1) {                                        
    clock_gettime(CLOCK_REALTIME, &now);
    timespec_add_us(&next, ps->period_us);

    fprintf(stderr,"th: %d, now is %ld, waking up at %ld\n", 
	   ps->index, now.tv_nsec, next.tv_nsec);

    if (timespec_cmp(&now, &next) > 0) {
      fprintf(stderr, "Deadline miss for thread %d\n", ps->index);
      fprintf(stderr, "now: %d sec %ld nsec     next: %d sec %ldnsec \n", 
	      now.tv_sec, now.tv_nsec, next.tv_sec, next.tv_nsec);
      exit(-1);
    }

    clock_nanosleep(CLOCK_REALTIME, TIMER_ABSTIME,
		    &next, NULL);

    fprintf(stderr,"th: %d started\n", ps->index);

    for (j=0; j<ps->wcet_sim; j++) a *= b;

    fprintf(stderr,"th: %d completed\n", ps->index);
  }
  return NULL;
}



int main(void) {
  pthread_t th[NTHREADS];
  pthread_attr_t myattr;
  int i;
  long periods[NTHREADS] = {300000, 200000, 100000};
  int wcets[NTHREADS] = {3000000, 2000000, 1000000};
  struct periodic_data_struct st[NTHREADS];
  struct sched_param myparam;
  
  fprintf(stderr,"Press 'q' to finish\n");

  pthread_attr_init(&myattr);   
  for (i=0; i<NTHREADS; i++) {
    st[i].index = i;
    st[i].period_us = periods[i];
    st[i].wcet_sim = wcets[i];

    pthread_attr_setschedpolicy(&myattr, SCHED_FIFO);
    myparam.sched_priority = i+10;
    pthread_attr_setschedparam(&myattr, &myparam);   
    pthread_create(&th[i], &myattr, thread_code, &st[i]);    
  }

  pthread_attr_destroy(&myattr);

  while (1) {
   if (getchar() == 'q') break;
  }
  return 0;
}

